@@ -204,7 +204,7 @@ static int net_listen(lua_State *L)
 	int n = lua_gettop(L);
 	if (n < 1 || n > 3) {
 		lua_error_p(L, "expected one to three arguments; usage:\n"
-				"net.listen(addressses, [port = " STR(KR_DNS_PORT)
+				"net.listen(addresses, [port = " STR(KR_DNS_PORT)
 				", flags = {tls = (port == " STR(KR_DNS_TLS_PORT) ")}])\n");
 	}
 
@@ -1092,7 +1092,7 @@ static int net_register_endpoint_kind(lua_State *L)
 	if (*pp != NULL || !strcasecmp(kind, "dns") || !strcasecmp(kind, "tls"))
 		lua_error_p(L, "attempt to register known kind '%s'\n", kind);
 	*pp = (char *)NULL + fun_id;
-	/* We don't attempt to engage correspoinding endpoints now.
+	/* We don't attempt to engage corresponding endpoints now.
 	 * That's the job for network_engage_endpoints() later. */
 	return 0;
 }
