@@ -328,6 +328,28 @@ local function dname2wire(name)
 	return ffi.string(name, knot.knot_dname_size(name))
 end
 
+-- Parse RDATA, from presentation to wire-format.
+-- in: a table of strings, each a line describing RRTYPE+RDATA
+-- out: a table of RDATA strings in wire-format
+local function parse_rdata(strs, nothing)
+	local zonefile = require('zonefile')
+	if type(strs) ~= 'table' or nothing ~= nil then -- accidents like forgetting braces
+		error('a table of string(s) is expected', 2)
+	end
+	local res = {}
+	for _, line in ipairs(strs) do
+		if type(line) ~= 'string' then
+			error('table must contain strings', 2)
+		end
+		local rrs = zonefile.string('. ' .. line)
+		if #rrs == 0 then error('failed to parse line: ' .. line, 2) end
+		for _, rr in ipairs(rrs) do
+			table.insert(res, rr.rdata)
+		end
+	end
+	return res
+end
+
 -- RR sets created in Lua must have a destructor to release allocated memory
 local function rrset_free(rr)
 	if rr._owner ~= nil then ffi.C.free(rr._owner) end
@@ -711,7 +733,7 @@ ffi.metatype( kr_query_t, {
 })
 
 -- helper for trace_chain_callbacks
--- ignores return values from successfull calls but logs tracebacks for throws
+-- ignores return values from successful calls but logs tracebacks for throws
 local function void_xpcall_log_tb(func, req, msg)
 	local ok, err = xpcall(func, debug.traceback, req, msg)
 	if not ok then
@@ -879,6 +901,11 @@ ffi.metatype( kr_request_t, {
 			req.vars_ref = ref
 			return var
 		end,
+		-- Ensure that answer has EDNS if needed; can't fail.
+		ensure_edns = function (req)
+			assert(ffi.istype(kr_request_t, req))
+			return C.kr_request_ensure_edns(req)
+		end,
 		-- Ensure that answer exists and return it; can't fail.
 		ensure_answer = function (req)
 			assert(ffi.istype(kr_request_t, req))
@@ -953,7 +980,8 @@ ffi.metatype( kr_cache_t, {
 				timestamp = tonumber(now.tv_sec)
 			end
 			-- Insert record into cache
-			local ret = C.kr_cache_insert_rr(self, rr, rrsig, tonumber(rank or 0), timestamp)
+			local ret = C.kr_cache_insert_rr(self, rr, rrsig, tonumber(rank or 0),
+											 timestamp, true)
 			if ret ~= 0 then return nil, knot_error_t(ret) end
 			return true
 		end,
@@ -1054,6 +1082,7 @@ kres = {
 	end,
 	dname2str = dname2str,
 	dname2wire = dname2wire,
+	parse_rdata = parse_rdata,
 
 	rr2str = rr2str,
 	str2ip = function (ip)
