@@ -9,7 +9,10 @@
 #include "daemon/tls.h"
 #include "daemon/worker.h"
 
-#include <assert.h>
+#if ENABLE_XDP
+	#include <libknot/xdp/eth.h>
+#endif
+
 #include <libgen.h>
 #include <net/if.h>
 #include <sys/un.h>
@@ -36,20 +39,18 @@ static int endpoint_open_lua_cb(struct network *net, struct endpoint *ep,
 				const char *log_addr)
 {
 	const bool ok = ep->flags.kind && !ep->handle && !ep->engaged && ep->fd != -1;
-	if (!ok) {
-		assert(!EINVAL);
+	if (kr_fails_assert(ok))
 		return kr_error(EINVAL);
-	}
 	/* First find callback in the endpoint registry. */
 	lua_State *L = the_worker->engine->L;
 	void **pp = trie_get_try(net->endpoint_kinds, ep->flags.kind,
 				strlen(ep->flags.kind));
 	if (!pp && net->missing_kind_is_error) {
-		kr_log_error("error: network socket kind '%s' not handled when opening '%s",
+		kr_log_error(NETWORK, "error: network socket kind '%s' not handled when opening '%s",
 				ep->flags.kind, log_addr);
 		if (ep->family != AF_UNIX)
-			kr_log_error("#%d", ep->port);
-		kr_log_error("'\n");
+			kr_log_error(NETWORK, "#%d", ep->port);
+		kr_log_error(NETWORK, "'\n");
 		return kr_error(ENOENT);
 	}
 	if (!pp) return kr_ok();
@@ -65,7 +66,7 @@ static int endpoint_open_lua_cb(struct network *net, struct endpoint *ep,
 		lua_pushfstring(L, "%s#%d", log_addr, ep->port);
 	}
 	if (lua_pcall(L, 3, 0, 0)) {
-		kr_log_error("error opening %s: %s\n", log_addr, lua_tostring(L, -1));
+		kr_log_error(NETWORK, "error opening %s: %s\n", log_addr, lua_tostring(L, -1));
 		return kr_error(ENOSYS); /* TODO: better value? */
 	}
 	ep->engaged = true;
@@ -105,7 +106,7 @@ static void endpoint_close_lua_cb(struct network *net, struct endpoint *ep)
 	void **pp = trie_get_try(net->endpoint_kinds, ep->flags.kind,
 				strlen(ep->flags.kind));
 	if (!pp && net->missing_kind_is_error) {
-		kr_log_error("internal error: missing kind '%s' in endpoint registry\n",
+		kr_log_error(NETWORK, "internal error: missing kind '%s' in endpoint registry\n",
 				ep->flags.kind);
 		return;
 	}
@@ -117,7 +118,7 @@ static void endpoint_close_lua_cb(struct network *net, struct endpoint *ep)
 	lua_pushpointer(L, ep);
 	lua_pushstring(L, "FIXME:endpoint-identifier");
 	if (lua_pcall(L, 3, 0, 0)) {
-		kr_log_error("failed to close FIXME:endpoint-identifier: %s\n",
+		kr_log_error(NETWORK, "failed to close FIXME:endpoint-identifier: %s\n",
 				lua_tostring(L, -1));
 	}
 }
@@ -134,14 +135,14 @@ static void endpoint_close(struct network *net, struct endpoint *ep, bool force)
 		socklen_t addr_len = sizeof(sa);
 		if (getsockname(ep->fd, (struct sockaddr *)&sa, &addr_len)
 		    || unlink(sa.sun_path)) {
-			kr_log_error("error (ignored) when closing unix socket (fd = %d): %s\n",
+			kr_log_error(NETWORK, "error (ignored) when closing unix socket (fd = %d): %s\n",
 					ep->fd, strerror(errno));
 			return;
 		}
 	}
 
 	if (ep->flags.kind && !is_control && !is_xdp) {
-		assert(!ep->handle);
+		kr_assert(!ep->handle);
 		/* Special lua-handled endpoint. */
 		if (ep->engaged) {
 			endpoint_close_lua_cb(net, ep);
@@ -154,7 +155,7 @@ static void endpoint_close(struct network *net, struct endpoint *ep, bool force)
 	}
 
 	free_const(ep->flags.kind); /* needed if (is_control) */
-	assert(ep->handle);
+	kr_require(ep->handle);
 	if (force) { /* Force close if event loop isn't running. */
 		if (ep->fd >= 0) {
 			close(ep->fd);
@@ -253,10 +254,8 @@ static int open_endpoint(struct network *net, const char *addr_str,
 		? sa == NULL && ep->fd == -1 && ep->nic_queue >= 0
 			&& ep->flags.sock_type == SOCK_DGRAM && !ep->flags.tls
 		: (sa != NULL) != (ep->fd != -1);
-	if (!ok) {
-		assert(!EINVAL);
+	if (kr_fails_assert(ok))
 		return kr_error(EINVAL);
-	}
 	if (ep->handle) {
 		return kr_error(EEXIST);
 	}
@@ -293,7 +292,7 @@ static int open_endpoint(struct network *net, const char *addr_str,
 	if (ep->family == AF_UNIX) {
 		/* Some parts of connection handling would need more work,
 		 * so let's support AF_UNIX only with .kind != NULL for now. */
-		kr_log_error("[system] AF_UNIX only supported with set { kind = '...' }\n");
+		kr_log_error(NETWORK, "AF_UNIX only supported with set { kind = '...' }\n");
 		ret = EAFNOSUPPORT;
 		goto finish_ret;
 		/*
@@ -314,10 +313,8 @@ static int open_endpoint(struct network *net, const char *addr_str,
 	} /* else */
 
 	if (ep->flags.sock_type == SOCK_DGRAM) {
-		if (ep->flags.tls) {
-			assert(!EINVAL);
+		if (kr_fails_assert(!ep->flags.tls))
 			return kr_error(EINVAL);
-		}
 		uv_udp_t *ep_handle = malloc(sizeof(uv_udp_t));
 		ep->handle = (uv_handle_t *)ep_handle;
 		ret = !ep->handle ? ENOMEM
@@ -334,7 +331,7 @@ static int open_endpoint(struct network *net, const char *addr_str,
 		goto finish_ret;
 	} /* else */
 
-	assert(!EINVAL);
+	kr_assert(false);
 	return kr_error(EINVAL);
 finish_ret:
 	if (!ret) return ret;
@@ -381,31 +378,25 @@ static int create_endpoint(struct network *net, const char *addr_str,
 
 int network_listen_fd(struct network *net, int fd, endpoint_flags_t flags)
 {
-	if (flags.xdp) {
-		assert(!EINVAL);
+	if (kr_fails_assert(!flags.xdp))
 		return kr_error(EINVAL);
-	}
 	/* Extract fd's socket type. */
 	socklen_t len = sizeof(flags.sock_type);
 	int ret = getsockopt(fd, SOL_SOCKET, SO_TYPE, &flags.sock_type, &len);
-	if (ret != 0) {
+	if (ret != 0)
 		return kr_error(errno);
-	}
-	if (flags.sock_type == SOCK_DGRAM && !flags.kind && flags.tls) {
-		assert(!EINVAL); /* Perhaps DTLS some day. */
-		return kr_error(EINVAL);
-	}
-	if (flags.sock_type != SOCK_DGRAM && flags.sock_type != SOCK_STREAM) {
+	const bool is_dtls = flags.sock_type == SOCK_DGRAM && !flags.kind && flags.tls;
+	if (kr_fails_assert(!is_dtls))
+		return kr_error(EINVAL);  /* Perhaps DTLS some day. */
+	if (flags.sock_type != SOCK_DGRAM && flags.sock_type != SOCK_STREAM)
 		return kr_error(EBADF);
-	}
 
 	/* Extract local address for this socket. */
 	struct sockaddr_storage ss = { .ss_family = AF_UNSPEC };
 	socklen_t addr_len = sizeof(ss);
 	ret = getsockname(fd, (struct sockaddr *)&ss, &addr_len);
-	if (ret != 0) {
+	if (ret != 0)
 		return kr_error(errno);
-	}
 
 	struct endpoint ep = {
 		.flags = flags,
@@ -458,10 +449,8 @@ static int16_t nic_queue_auto(void)
 int network_listen(struct network *net, const char *addr, uint16_t port,
 		   int16_t nic_queue, endpoint_flags_t flags)
 {
-	if (net == NULL || addr == 0 || nic_queue < -1) {
-		assert(!EINVAL);
+	if (kr_fails_assert(net != NULL && addr != 0 && nic_queue >= -1))
 		return kr_error(EINVAL);
-	}
 
 	if (flags.xdp && nic_queue < 0) {
 		nic_queue = nic_queue_auto();
@@ -560,9 +549,9 @@ void network_new_hostname(struct network *net, struct engine *engine)
 		if (newcreds) {
 			tls_credentials_release(net->tls_credentials);
 			net->tls_credentials = newcreds;
-			kr_log_info("[tls] Updated ephemeral X.509 cert with new hostname\n");
+			kr_log_info(TLS, "Updated ephemeral X.509 cert with new hostname\n");
 		} else {
-			kr_log_error("[tls] Failed to update ephemeral X.509 cert with new hostname, using existing one\n");
+			kr_log_error(TLS, "Failed to update ephemeral X.509 cert with new hostname, using existing one\n");
 		}
 	}
 }
@@ -571,16 +560,16 @@ void network_new_hostname(struct network *net, struct engine *engine)
 static int set_bpf_cb(const char *key, void *val, void *ext)
 {
 	endpoint_array_t *endpoints = (endpoint_array_t *)val;
-	assert(endpoints != NULL);
 	int *bpffd = (int *)ext;
-	assert(bpffd != NULL);
+	if (kr_fails_assert(endpoints && bpffd))
+		return kr_error(EINVAL);
 
 	for (size_t i = 0; i < endpoints->len; i++) {
 		struct endpoint *endpoint = &endpoints->at[i];
 		uv_os_fd_t sockfd = -1;
 		if (endpoint->handle != NULL)
 			uv_fileno(endpoint->handle, &sockfd);
-		assert(sockfd != -1);
+		kr_require(sockfd != -1);
 
 		if (setsockopt(sockfd, SOL_SOCKET, SO_ATTACH_BPF, bpffd, sizeof(int)) != 0) {
 			return 1; /* return error (and stop iterating over net->endpoints) */
@@ -599,7 +588,7 @@ int network_set_bpf(struct network *net, int bpf_fd)
 		return 0;
 	}
 #else
-	kr_log_error("[network] SO_ATTACH_BPF socket option doesn't supported\n");
+	kr_log_error(NETWORK, "SO_ATTACH_BPF socket option doesn't supported\n");
 	(void)net;
 	(void)bpf_fd;
 	return 0;
@@ -611,17 +600,18 @@ int network_set_bpf(struct network *net, int bpf_fd)
 static int clear_bpf_cb(const char *key, void *val, void *ext)
 {
 	endpoint_array_t *endpoints = (endpoint_array_t *)val;
-	assert(endpoints != NULL);
+	if (kr_fails_assert(endpoints))
+		return kr_error(EINVAL);
 
 	for (size_t i = 0; i < endpoints->len; i++) {
 		struct endpoint *endpoint = &endpoints->at[i];
 		uv_os_fd_t sockfd = -1;
 		if (endpoint->handle != NULL)
 			uv_fileno(endpoint->handle, &sockfd);
-		assert(sockfd != -1);
+		kr_require(sockfd != -1);
 
 		if (setsockopt(sockfd, SOL_SOCKET, SO_DETACH_BPF, NULL, 0) != 0) {
-			kr_log_error("[network] failed to clear SO_DETACH_BPF socket option\n");
+			kr_log_error(NETWORK, "failed to clear SO_DETACH_BPF socket option\n");
 		}
 		/* Proceed even if setsockopt() failed,
 		 * as we want to process all opened sockets. */
@@ -635,7 +625,7 @@ void network_clear_bpf(struct network *net)
 #ifdef SO_DETACH_BPF
 	map_walk(&net->endpoints, clear_bpf_cb, NULL);
 #else
-	kr_log_error("[network] SO_DETACH_BPF socket option doesn't supported\n");
+	kr_log_error(NETWORK, "SO_DETACH_BPF socket option doesn't supported\n");
 	(void)net;
 #endif
 }
