@@ -12,7 +12,6 @@
 
 struct qr_task;
 
-#include <assert.h>
 #include <sys/socket.h>
 
 
@@ -102,7 +101,7 @@ int udp_queue_init_global(uv_loop_t *loop)
 void udp_queue_push(int fd, struct kr_request *req, struct qr_task *task)
 {
 	if (fd < 0) {
-		kr_log_error("ERROR: called udp_queue_push(fd = %d, ...)\n", fd);
+		kr_log_error(SYSTEM, "ERROR: called udp_queue_push(fd = %d, ...)\n", fd);
 		abort();
 	}
 	worker_task_ref(task);
@@ -134,7 +133,7 @@ void udp_queue_push(int fd, struct kr_request *req, struct qr_task *task)
 	++(q->len);
 
 	if (q->len >= UDP_QUEUE_LEN) {
-		assert(q->len == UDP_QUEUE_LEN);
+		kr_assert(q->len == UDP_QUEUE_LEN);
 		udp_queue_send(fd);
 		/* We don't need to search state.waiting_fds;
 		 * anyway, it's more efficient to let the hook do that. */
