@@ -9,17 +9,14 @@
 
 #include "lib/cache/impl.h"
 
-int rdataset_dematerialize(const knot_rdataset_t *rds, uint8_t * restrict data)
+void rdataset_dematerialize(const knot_rdataset_t *rds, uint8_t * restrict data)
 {
 	/* FIXME: either give up on even alignment and thus direct usability
 	 * of rdatasets as they are in lmdb, or align inside cdb_* functions
 	 * (request sizes one byte longer and shift iff on an odd address). */
 	//if ((size_t)data & 1) VERBOSE_MSG(NULL, "dematerialize: odd address\n");
 	//const uint8_t *data0 = data;
-	if (!data) {
-		assert(data);
-		return kr_error(EINVAL);
-	}
+	kr_require(data);
 	const uint16_t rr_count = rds ? rds->count : 0;
 	memcpy(data, &rr_count, sizeof(rr_count));
 	data += sizeof(rr_count);
@@ -28,8 +25,7 @@ int rdataset_dematerialize(const knot_rdataset_t *rds, uint8_t * restrict data)
 		data += rds->size;
 	}
 	//VERBOSE_MSG(NULL, "dematerialized to %d B\n", (int)(data - data0));
-	(void)data;
-	return kr_ok();
+	(void)data; // silence analyzers
 }
 
 /** Materialize a knot_rdataset_t from cache with given TTL.
@@ -38,9 +34,10 @@ int rdataset_dematerialize(const knot_rdataset_t *rds, uint8_t * restrict data)
 static int rdataset_materialize(knot_rdataset_t * restrict rds, const uint8_t * const data,
 				const uint8_t *data_bound, knot_mm_t *pool)
 {
-	assert(rds && data && data_bound && data_bound > data && !rds->rdata
-		/*&& !((size_t)data & 1)*/);
-	assert(pool); /* not required, but that's our current usage; guard leaks */
+	if (kr_fails_assert(rds && data && data_bound && data_bound > data && !rds->rdata
+		       /*&& !((size_t)data & 1)*/))
+		return kr_error(EINVAL);
+	kr_assert(pool); /* not required, but that's our current usage; guard leaks */
 	const uint8_t *d = data; /* iterates over the cache data */
 	/* First sum up the sizes for wire format length. */
 	/* TODO: we might overrun here already, but we need to trust cache anyway...*/
@@ -77,23 +74,18 @@ int entry2answer(struct answer *ans, int id,
 		const knot_dname_t *owner, uint16_t type, uint32_t new_ttl)
 {
 	/* We assume it's zeroed.  Do basic sanity check. */
-	if (ans->rrsets[id].set.rr || ans->rrsets[id].sig_rds.rdata
+	const bool not_ok = ans->rrsets[id].set.rr || ans->rrsets[id].sig_rds.rdata
 	    || (type == KNOT_RRTYPE_NSEC  &&  ans->nsec_p.raw)
-	    || (type == KNOT_RRTYPE_NSEC3 && !ans->nsec_p.raw)
-	   )
-	{
-		assert(false);
+	    || (type == KNOT_RRTYPE_NSEC3 && !ans->nsec_p.raw);
+	if (kr_fails_assert(!not_ok))
 		return kr_error(EINVAL);
-	}
 	/* Materialize the base RRset. */
 	knot_rrset_t *rr = ans->rrsets[id].set.rr
 		= knot_rrset_new(owner, type, KNOT_CLASS_IN, new_ttl, ans->mm);
-	if (!rr) {
-		assert(!ENOMEM);
+	if (kr_fails_assert(rr))
 		return kr_error(ENOMEM);
-	}
 	int ret = rdataset_materialize(&rr->rrs, eh->data, eh_bound, ans->mm);
-	if (ret < 0) goto fail;
+	if (kr_fails_assert(ret >= 0)) goto fail;
 	size_t data_off = ret;
 	ans->rrsets[id].set.rank = eh->rank;
 	ans->rrsets[id].set.expiring = is_expiring(eh->ttl, new_ttl);
@@ -102,20 +94,18 @@ int entry2answer(struct answer *ans, int id,
 	if (want_rrsigs) {
 		ret = rdataset_materialize(&ans->rrsets[id].sig_rds, eh->data + data_off,
 					   eh_bound, ans->mm);
-		if (ret < 0) goto fail;
+		if (kr_fails_assert(ret >= 0)) goto fail;
 		/* Sanity check: we consumed exactly all data. */
 		int unused_bytes = eh_bound - (uint8_t *)eh->data - data_off - ret;
-		if (unused_bytes) {
-			kr_log_error("[cach] entry2answer ERROR: unused bytes: %d\n",
+		if (kr_fails_assert(unused_bytes == 0)) {
+			kr_log_error(CACHE, "entry2answer ERROR: unused bytes: %d\n",
 					unused_bytes);
-			assert(!EILSEQ);
 			ret = kr_error(EILSEQ);
 			goto fail; /* to be on the safe side */
 		}
 	}
 	return kr_ok();
 fail:
-	assert(/*false*/!ret);
 	/* Cleanup the item that we might've (partially) written to. */
 	knot_rrset_free(ans->rrsets[id].set.rr, ans->mm);
 	knot_rdataset_clear(&ans->rrsets[id].sig_rds, ans->mm);
