@@ -1,4 +1,4 @@
 /* SPDX-License-Identifier: GPL-3.0-or-later */
-#include <knot/lib/pkt.h>
+#include <libknot/packet/pkt.h>
 
 uint32_t packet_ttl(const knot_pkt_t *pkt, bool is_negative);
