@@ -8,21 +8,7 @@
 #include "lib/defines.h"
 #include "lib/utils.h"
 
-#ifdef NOVERBOSELOG
-	#define QRVERBOSE(query, cls, ...)
-#else
-	/** Print a debug message related to resolution.
-	 * \param _query	associated kr_query, may be NULL
-	 * \param _cls	identifying string, typically of length exactly four (padded)
-	 * \param ...	printf-compatible list of parameters
-	 */
-	#define QRVERBOSE(_query, _cls, ...) do { \
-		const struct kr_query *_qry = (_query); \
-		if (VERBOSE_STATUS || kr_log_qtrace_enabled(_qry)) { \
-			kr_log_q(_qry, (_cls), __VA_ARGS__); \
-		}  \
-	} while (false)
-#endif
+#define QRVERBOSE kr_log_q // TODO: perhaps eliminate
 
 /** Layer processing states.  Only one value at a time (but see TODO).
  *
@@ -51,6 +37,7 @@ static inline bool kr_state_consistent(enum kr_layer_state s)
 #ifdef __clang__
 #pragma clang diagnostic push
 #pragma clang diagnostic ignored "-Wtautological-constant-out-of-range-compare"
+#pragma clang diagnostic ignored "-Wtautological-unsigned-enum-zero-compare"
 #endif
 	return s >= 0 && s < (1 << 5);
 #ifdef __clang__
