@@ -10,7 +10,7 @@
 #include "lib/zonecut.h"
 #include "lib/resolve.h"
 
-#define VERBOSE_MSG(qry, ...) QRVERBOSE((qry), "slct", __VA_ARGS__)
+#define VERBOSE_MSG(qry, ...) QRVERBOSE((qry), SELECTION, __VA_ARGS__)
 
 /// To be held per query and locally.  Allocations are in the kr_request's mempool.
 struct iter_local_state {
@@ -48,7 +48,7 @@ static struct address_state *get_address_state(struct iter_local_state *local_st
 	trie_val_t *address_state = trie_get_try(local_state->addresses, (char *)address,
 						 transport->address_len);
 	if (!address_state) {
-		assert(transport->deduplicated);
+		kr_assert(transport->deduplicated);
 		/* Transport was chosen by a different query. */
 		return NULL;
 	}
@@ -127,6 +127,7 @@ static void unpack_state_from_zonecut(struct iter_local_state *local_state,
 		}
 	}
 	trie_it_free(it);
+	kr_cache_commit(&qry->request->ctx->cache);
 }
 
 static int get_valid_addresses(struct iter_local_state *local_state,
@@ -227,7 +228,7 @@ static void update_name_state(knot_dname_t *name, enum kr_transport_protocol typ
 		name_state->aaaa_state = RECORD_TRIED;
 		break;
 	default:
-		assert(0);
+		kr_assert(false);
 	}
 }
 
@@ -281,7 +282,7 @@ void iter_choose_transport(struct kr_query *qry, struct kr_transport **transport
 			local_state->no_ns_addr_count = 0;
 			break;
 		default:
-			assert(0);
+			kr_assert(false);
 			break;
 		}
 
@@ -302,7 +303,7 @@ void iter_choose_transport(struct kr_query *qry, struct kr_transport **transport
 		qry->flags.DNSSEC_BOGUS = true;
 	}
 
-	WITH_VERBOSE(qry)
+	if (kr_log_is_debug_qry(SELECTION, qry))
 	{
 	KR_DNAME_GET_STR(zonecut_str, qry->zone_cut.name);
 	if (*transport) {
