@@ -66,7 +66,7 @@ typedef void (*trace_log_f)(const struct kr_request *request, const char *msg);
 /** Whether kr_assert() and kr_fails_assert() checks should abort. */
 KR_EXPORT extern bool kr_dbg_assertion_abort;
 
-/** How often kr_asert() should fork the process before issuing abort (if configured).
+/** How often kr_assert() should fork the process before issuing abort (if configured).
  *
  * This can be useful for debugging rare edge-cases in production.
  * if (kr_debug_assertion_abort && kr_debug_assertion_fork), it is
@@ -104,8 +104,8 @@ static inline bool kr_assert_func(bool result, const char *expr, const char *fun
 	knot_rrtype_to_string((rrtype), rrtype_str, sizeof(rrtype_str)); \
 	rrtype_str[sizeof(rrtype_str) - 1] = 0;
 
-// Use this for alocations with mm.
-// Use mm_alloc for alocations into mempool
+// Use this for allocations with mm.
+// Use mm_alloc for allocations into mempool
 
 /** A strcmp() variant directly usable for qsort() on an array of strings. */
 static inline int strcmp_p(const void *p1, const void *p2)
@@ -113,21 +113,6 @@ static inline int strcmp_p(const void *p1, const void *p2)
 	return strcmp(*(char * const *)p1, *(char * const *)p2);
 }
 
-
-/** Return time difference in miliseconds.
-  * @note based on the _BSD_SOURCE timersub() macro */
-static inline long time_diff(struct timeval *begin, struct timeval *end) {
-    struct timeval res = {
-        .tv_sec = end->tv_sec - begin->tv_sec,
-        .tv_usec = end->tv_usec - begin->tv_usec
-    };
-    if (res.tv_usec < 0) {
-        --res.tv_sec;
-        res.tv_usec += 1000000;
-    }
-    return res.tv_sec * 1000 + res.tv_usec / 1000;
-}
-
 /** Get current working directory with fallback value. */
 static inline void get_workdir(char *out, size_t len) {
 	if(getcwd(out, len) == NULL) {
@@ -298,7 +283,7 @@ int kr_ntop_str(int family, const void *src, uint16_t port, char *buf, size_t *b
 static inline char *kr_straddr(const struct sockaddr *addr)
 {
 	if (kr_fails_assert(addr)) return NULL;
-	/* We are the sinle-threaded application */
+	/* We are the single-threaded application */
 	static char str[INET6_ADDRSTRLEN + 1 + 5 + 1];
 	size_t len = sizeof(str);
 	int ret = kr_inaddr_str(addr, str, &len);
@@ -401,6 +386,11 @@ int kr_ranked_rrarray_set_wire(ranked_rr_array_t *array, bool to_wire,
 			       uint32_t qry_uid, bool check_dups,
 			       bool (*extraCheck)(const ranked_rr_array_entry_t *));
 
+
+/** Style used by the kr_*_text() functions. */
+KR_EXPORT extern
+const knot_dump_style_t KR_DUMP_STYLE_DEFAULT;
+
 /**
  * @return Newly allocated string representation of packet.
  * Caller has to free() returned string.
@@ -492,6 +482,43 @@ static inline int kr_dname_lf(uint8_t *dst, const knot_dname_t *src, bool add_wi
 	return KNOT_EOK;
 }
 
+
+/** Timer, i.e stop-watch. */
+typedef struct timespec kr_timer_t;
+
+/** Start, i.e. set the reference point. */
+static inline void kr_timer_start(kr_timer_t *start)
+{
+	/* The call should be very reliable, but let's check it in _start() at least. */
+	kr_require(start && clock_gettime(CLOCK_MONOTONIC, start) == 0);
+}
+
+/** Get elapsed time in floating-point seconds. */
+static inline double kr_timer_elapsed(kr_timer_t *start)
+{
+	kr_require(start);
+	kr_timer_t end = { 0 };
+	(void)clock_gettime(CLOCK_MONOTONIC, &end);
+	return (end.tv_sec - start->tv_sec) + (double)(end.tv_nsec - start->tv_nsec) / 1e9;
+}
+
+/** Get elapsed time in micro-seconds. */
+static inline uint64_t kr_timer_elapsed_us(kr_timer_t *start)
+{
+	kr_require(start);
+	kr_timer_t end = { 0 };
+	(void)clock_gettime(CLOCK_MONOTONIC, &end);
+	// avoid negative differences, because of integer division
+	if (end.tv_nsec - start->tv_nsec < 0) {
+		end.tv_nsec += 1000*1000*1000;
+		end.tv_sec  -= 1;
+	}
+	return (uint64_t)(end.tv_sec - start->tv_sec) * 1000000
+		// adding 500 gives us rounding
+		+ (end.tv_nsec - start->tv_nsec + 500) / 1000;
+}
+
+
 /**
  * Difference between two calendar times specified as strings.
  * \param[in]  format format for strptime
