@@ -1,3 +1,4 @@
+-- SPDX-License-Identifier: GPL-3.0-or-later
 -- Speculative prefetching for repetitive and soon-expiring records to reduce latency.
 -- @module predict
 -- @field queue queue of scheduled queries
@@ -158,7 +159,10 @@ end
 
 function predict.config(config)
 	-- Reconfigure
-	if type(config) ~= 'table' then return end
+	config = config or {}
+	if type(config) ~= 'table' then
+		error('[predict] configuration must be a table or nil')
+	end
 	if config.window then predict.window = config.window end
 	if config.period then predict.period = config.period end
 	-- Load dependent modules
@@ -172,7 +176,6 @@ predict.layer = {
 	-- Prefetch all expiring (sub-)queries immediately after the request finishes.
 	-- Doing that immediately is simplest and avoids creating (new) large bursts of activity.
 	finish = function (_, req)
-		req = kres.request_t(req)
 		local qrys = req.rplan.resolved
 		for i = 0, (tonumber(qrys.len) - 1) do -- size_t doesn't work for some reason
 			local qry = qrys.at[i]
