@@ -50,7 +50,7 @@ function parse_pkt(input, desc)
 end
 
 local function check_ok(req, desc)
-	local headers, stream, errno = req:go(8)  -- randomly chosen timeout by tkrizek
+	local headers, stream, errno = req:go(16)
 	if errno then
 		local errmsg = stream
 		nok(errmsg, desc .. ': ' .. errmsg)
@@ -64,7 +64,7 @@ local function check_ok(req, desc)
 end
 
 --local function check_err(req, exp_status, desc)
---	local headers, errmsg, errno = req:go(8)  -- randomly chosen timeout by tkrizek
+--	local headers, errmsg, errno = req:go(16)
 --	if errno then
 --		nok(errmsg, desc .. ': ' .. errmsg)
 --		return
@@ -285,7 +285,7 @@ else
 --		local req = assert(req_templ:clone())
 --		req.headers:upsert(':method', 'GET')
 --		req.headers:upsert(':path', '/doh?notdns=' .. basexx.to_url64(string.rep('\0', 1024)))
---		check_err(req, '400', 'GET without dns paramter finishes with 400')
+--		check_err(req, '400', 'GET without dns parameter finishes with 400')
 --	end
 --
 --	local function test_get_unparseable()
